@@ -41,7 +41,7 @@ char *imap_read_server_capacity(int sock) {
 
 int start_imap(int s, char *ip, int port, unsigned char options, char *miscptr, FILE * fp) {
   char *empty = "";
-  char *login, *pass, buffer[500], buffer2[500];
+  char *login, *pass, buffer[500], buffer2[500], *fooptr;
 
   if (strlen(login = hydra_get_next_login()) == 0)
     login = empty;
@@ -198,13 +198,14 @@ int start_imap(int s, char *ip, int port, unsigned char options, char *miscptr,
       from64tobits((char *) buffer, buf);
       free(buf);
 
-      if (verbose)
+      if (debug)
         hydra_report(stderr, "DEBUG S: %s\n", buffer);
 
-      sasl_digest_md5(buffer2, login, pass, buffer, miscptr, "imap", NULL, 0, NULL);
-      if (buffer2 == NULL)
+      fooptr = buffer2;
+      sasl_digest_md5(fooptr, login, pass, buffer, miscptr, "imap", NULL, 0, NULL);
+      if (fooptr == NULL)
         return 3;
-      if (verbose)
+      if (debug)
         hydra_report(stderr, "DEBUG C: %s\n", buffer2);
       hydra_tobase64((unsigned char *) buffer2, strlen(buffer2), sizeof(buffer2));
       sprintf(buffer, "%s\r\n", buffer2);
@@ -261,8 +262,9 @@ int start_imap(int s, char *ip, int port, unsigned char options, char *miscptr,
         serverfirstmessage[sizeof(serverfirstmessage) - 1] = '\0';
 
         memset(buffer2, 0, sizeof(buffer2));
-        sasl_scram_sha1(buffer2, pass, clientfirstmessagebare, serverfirstmessage);
-        if (buffer2 == NULL) {
+        fooptr = buffer2;
+        sasl_scram_sha1(fooptr, pass, clientfirstmessagebare, serverfirstmessage);
+        if (fooptr == NULL) {
           hydra_report(stderr, "[ERROR] Can't compute client response\n");
           return 1;
         }
@@ -299,7 +301,11 @@ int start_imap(int s, char *ip, int port, unsigned char options, char *miscptr,
       if (hydra_send(s, buffer, strlen(buffer), 0) < 0)
         return 1;
       if ((buf = hydra_receive_line(s)) == NULL)
-        return (1);
+        return 1;
+      if (strlen(buf) < 6) {
+        free(buf);
+        return 1;
+      }
 
       //recover challenge
       from64tobits((char *) buf1, buf + 2);
